@@ -1,6 +1,6 @@
 use super::grammars::{InputGrammar, PrecedenceEntry, Variable, VariableType};
 use super::rules::{Precedence, Rule};
-use crate::error::{Error, Result};
+use anyhow::{anyhow, Result};
 use serde_derive::Deserialize;
 use serde_json::{Map, Value};
 
@@ -109,9 +109,8 @@ pub(crate) fn parse_grammar(input: &str) -> Result<InputGrammar> {
                 RuleJSON::STRING { value } => PrecedenceEntry::Name(value),
                 RuleJSON::SYMBOL { name } => PrecedenceEntry::Symbol(name),
                 _ => {
-                    return Err(Error::new(
+                    return Err(anyhow!(
                         "Invalid rule in precedences array. Only strings and symbols are allowed"
-                            .to_string(),
                     ))
                 }
             })
